module Microservice
  class ApplicationController < ActionController::Base

    protect_from_forgery with: :exception

    layout 'microservice/application'

    class_attribute :allow_custom_type
    class_attribute :entity_class_for_link

    around_action :set_current_user

    protected

    def render_403(options = {})
      render_error({message: :notice_not_authorized, status: 403}.merge(options))
      return false
    end

    def render_404(options = {})
      render_error({message: :notice_entity_not_found, status: 404}.merge(options))
      return false
    end

    def render_validation_errors(errors)
      @errors = errors
      render template: 'mini_easy/common/error', formats: params[:format]
    end

    def render_entity_validation_errors(entity)
      render_validation_errors(entity.errors.full_messages)
    end

    def render_error(arg)
      arg = {message: arg} if !arg.is_a?(Hash)

      message = arg[:message]
      message = I18n.t(message) if message.is_a?(Symbol)

      @errors = [message]
      @status = arg[:status] || 500

      render template: 'common/error', status: @status, formats: params[:format]
    end

    def set_current_user
      User.current = current_user
      yield
    ensure
      User.current = nil
    end

    def current_user_ip
      request.env['HTTP_X_FORWARDED_FOR'].blank? ? request.remote_ip : request.env['HTTP_X_FORWARDED_FOR'].to_s.split(',').first
    end

  end
end
