module Microservice
  class ApplicationController < ::Microservice::ApplicationController

    class_attribute :entity_class

    before_action :find_entity_class, only: [:new, :create]
    before_action :find_entity, only: [:show, :edit, :update, :destroy]
    before_action :find_copy_from, only: [:new, :create]
    before_action :create_entity, only: [:new, :create]
    before_action :find_parent_entity, only: [:index, :show, :new, :create, :edit, :update, :destroy]
    before_action :load_entities, only: [:index]
    before_action :assign_attributes, only: [:new, :create, :edit, :update]

    def index
      render layout: !request.xhr?
    end

    def show
      render layout: !request.xhr?
    end

    def new
      render layout: !request.xhr?
    end

    def create
      if @entity.save
        respond_to do |format|
          format.html {
            flash[:notice] = t(:notice_successful_create)
            redirect_to polymorphic_path(@entity.model_route_array)
          }
          format.js
        end
      else
        respond_to do |format|
          format.html {render action: 'new', layout: !request.xhr?}
          format.js
        end
      end
    end

    def edit
      render layout: !request.xhr?
    end

    def update
      if @entity.save
        respond_to do |format|
          format.html {
            flash[:notice] = t(:notice_successful_update)
            redirect_to polymorphic_path(@entity.model_route_array)
          }
          format.js
        end
      else
        respond_to do |format|
          format.html {render action: 'edit', layout: !request.xhr?}
          format.js
        end
      end
    end

    def destroy
      @entity.destroy

      respond_to do |format|
        format.html {
          flash[:notice] = t(:notice_successful_delete)
          redirect_to polymorphic_path(entity_class.class_route_array)
        }
        format.js
      end
    end

    def search
      render json: self.entity_class.search(params[:q])
    end

    protected

    def find_entity_class
      @current_entity_class ||=
        if allow_custom_type && params[:type]
          params[:type].safe_constantize
        elsif !allow_custom_type
          self.entity_class
        end
      render_404 if !@current_entity_class
    rescue NameError
      render_404
    end

    def entity_symbol
      self.entity_class.name.underscore.to_sym
    end

    def find_entity
      @entity = self.entity_class.find(params[:id])
      @parent_entity = @entity.parent_entity
    rescue ActiveRecord::RecordNotFound
      render_404
    end

    def find_parent_entity
      @parent_entity = self.entity_class.parent_entity_class.find(params[self.entity_class.parent_entity_id_symbol]) if self.entity_class.parent_entity_class
    rescue ActiveRecord::RecordNotFound
      render_404
    end

    def find_copy_from
      @copy_from = @current_entity_class.find(params[:copy_from]) if params[:copy_from]
    rescue ActiveRecord::RecordNotFound
      render_404
    end

    def create_entity
      @entity = @current_entity_class.copy_from(@copy_from) if @copy_from
      @entity ||= @current_entity_class.new
    end

    def load_entities
      @entities = self.entity_class.sorted
    end

    def entity_params
      params.require(entity_symbol).permit([])
    end

    def assign_attributes
      @entity.parent_entity ||= @parent_entity if @parent_entity
      @entity.attributes = entity_params if params[entity_symbol]
      @entity.author = current_user if @entity.respond_to?(:author_id) && @entity.new_record?
      @entity.updated_by = current_user if @entity.respond_to?(:updated_by_id) && !@entity.new_record?
      end

  end
end
