module Microservice
  module BreadcrumbsHelper

    def render_breadcrumb(*args)
      flatten_args = args.flatten.compact
      content_tag :div, class: 'ui huge breadcrumb' do
        flatten_args.map.with_index do |element, idx|
          if flatten_args.size == idx + 1
            content_tag(:span, element.html_safe, class: 'section active')
          else
            content_tag(:span, element.html_safe, class: 'section') +
              content_tag(:i, '', class: 'right chevron icon divider')
          end
        end.join.html_safe
      end
    end

    def render_entity_breadcrumb
      if @entity
        case action_name
        when 'show'
          render_breadcrumb_show @entity
        when 'new', 'create'
          render_breadcrumb_new @entity
        when 'edit', 'update'
          render_breadcrumb_edit @entity
        end
      elsif controller.entity_class
        render_breadcrumb_index controller.entity_class
      end
    end

    def render_breadcrumb_index(entity_class)
      links = [home_link(entity_class.namespace)]

      if @parent_entity
        links << link_to_entity(@parent_entity.class, :index, no_icon: true)
        links << link_to_entity(@parent_entity, :show, no_icon: true)
      end

      links << link_to_entity(entity_class, :index, no_icon: true)

      render_breadcrumb links
    end

    def render_breadcrumb_show(entity)
      links = [home_link(entity.namespace)]

      if @parent_entity
        links << link_to_entity(@parent_entity.class, :index, no_icon: true)
        links << link_to_entity(@parent_entity, :show, no_icon: true)
      end

      links << link_to_entity(entity.class, :index, no_icon: true)
      links << link_to_entity(entity, :show, no_icon: true)

      render_breadcrumb links
    end

    def render_breadcrumb_new(entity)
      links = [home_link(entity.namespace)]

      if @parent_entity
        links << link_to_entity(@parent_entity.class, :index, no_icon: true)
        links << link_to_entity(@parent_entity, :show, no_icon: true)
      end

      links << link_to_entity(entity.class, :index, no_icon: true)
      links << link_to_entity(entity.class, :new, no_icon: true)

      render_breadcrumb links
    end

    def render_breadcrumb_edit(entity)
      links = [home_link(entity.namespace)]

      if @parent_entity
        links << link_to_entity(@parent_entity.class, :index, no_icon: true)
        links << link_to_entity(@parent_entity, :show, no_icon: true)
      end

      links << link_to_entity(entity.class, :index, no_icon: true)
      links << link_to_entity(entity, :edit, no_icon: true)

      render_breadcrumb links
    end

  end
end
