module Microservice
  module Concerns
    module EntityCloneable
      extend ActiveSupport::Concern

      included do

        after_create :copy_associations, if: -> {self.copied_from.present?}

        attr_accessor :copied_from, :copy_options

      end

      class_methods do

        def copy_from(arg, options = {})
          source = arg.is_a?(::ApplicationRecord) ? arg : self.class.find(arg)

          copy = self.new
          copy.attributes = source.attributes.dup.except(*copy_attributes_exceptions)
          copy.copied_from = source
          copy.copy_options = options

          copy
        end

        def copy_attributes_exceptions
          %w(id created_at updated_at)
        end

      end

      protected

      def copy_associations
      end

    end
  end
end
